
#include <avr/io.h>
//#include <avr/interrupt.h>
#include <inttypes.h>
#define MHZ *1000000UL
#define F_CPU (20 MHZ )  // 1 MHz
#include <util/delay.h>
#include <avr/pgmspace.h>


void delay_ms(unsigned int ms)
/* delay for a minimum of <ms> */
{
  // we use a calibrated macro. This is more
  // accurate and not so much compiler dependent
  // as self made code.
  while(ms){
    _delay_ms(0.96);
    ms--;
  }
}

#define PB (1<<3)
#define PC (2<<3)
#define PD (3<<3)

void clr_bit (unsigned char b)
{
  unsigned char bv;
  bv = 1 << (b&7);
  b &= ~7;
  switch (b) {
  case PB:PORTB &= ~bv;break;
  case PC:PORTC &= ~bv;break;
  case PD:PORTD &= ~bv;break;
  }
}


void set_bit (unsigned char b)
{
  unsigned char bv;
  bv = 1 << (b&7);
  b &= ~7;
  switch (b) {
  case PB:PORTB |= bv;break;
  case PC:PORTC |= bv;break;
  case PD:PORTD |= bv;break;
  }
}

void make_output (unsigned char b)
{
  unsigned char bv;
  bv = 1 << (b&7);
  b &= ~7;
  switch (b) {
  case PB:DDRB |= bv;break;
  case PC:DDRC |= bv;break;
  case PD:DDRD |= bv;break;
  }
  
}

#ifndef USBPOWER

#if defined (__AVR_ATmega168__) 
#define FTDI_ATMEGA
#elif defined (__AVR_AT90USB162__)
#define USBIO
#else
#error do not know what outputs to define.
#endif

#endif

#ifdef FTDI_ATMEGA
unsigned char leds [] = 
  { PC | PC3, PC | PC2, PC | PC1, PC | PC0, 
    PB | PB5, PB | PB4, PB | PB3, PB | PB2, 
    PB | PB1, PB | PB0, PD | PD7, PD | PD6, 
    PD | PD5, PD | PD4, PD | PD3, PD | PD2};
#define LEDSOK
#endif

#ifdef USBIO
unsigned char leds [] = 
  { PC | PC7, PC | PC6, PB | PB6, PB | PB7, 
    PB | PB4, PB | PB5, PB | PB2, PB | PB3, 
    PB | PB0, PB | PB1, PD | PD6, PD | PD5, 
    PD | PD4, PD | PD3, PD | PD2, PD | PD1};
#define LEDSOK
#endif

#ifdef USBPOWER
unsigned char leds [] = 
  { PD | PD6, PD | PD5, PD | PD4, PD | PD3,
    PD | PD2, PD | PD1, PD | PD0 };
#define LEDSOK
#endif

#ifndef LEDSOK
#warning You have to define one of the boards. 
#endif


int main (int argc, char **argv)
{
  unsigned char l;
  //  unsigned char v;
  unsigned char up;
  unsigned char t;

  for (l=0;l<sizeof(leds);l++) {
    make_output (leds[l]);
  }

  DDRC |= _BV(PC5);
  t = 0;
  up = 1;
  l=0;
  while (1) {
    t++;
    //    if (t&16) set_bit (PC | PC5);
    //    else      clr_bit (PC | PC5);
    set_bit (leds[l]);
    delay_ms (200);
    clr_bit (leds[l]);
    if (up) {
      l++;
      if ((l+1) == sizeof (leds)) 
	up = 0;
    } else {
      l--;
      if (l == 0) 
	up = 1;
    }
  }
}
